#!/usr/bin/perl -w
#Filename:
#Author: Li Cuiping
#Date: 2022-06-03
#Modified:
#Description: ﶻ߱ע.
my $version=1.00;

use strict;
use Getopt::Long;
use File::Basename;

my %opts;
GetOptions(\%opts,"r=s","refdir=s","i=s","dir=s","h");
if (!(defined $opts{i} and defined $opts{dir} and defined $opts{r} and defined $opts{refdir}) || defined $opts{h}) { #necessary arguments
&usage;
}

my $reftag=1;
my $ref_file=$opts{r};
$reftag=0;


open R,"<$ref_file";
my $ref_content=do {local $/=undef; <R>;};
my @ref_line=split /\n/,$ref_content;
my $ref_seq=join "",@ref_line[1..$#ref_line];
my $ref_length=length($ref_seq);
my @ref_head=split /\s+/,$ref_line[0];
$ref_head[0]=~s/>//;
my $ref_chr=$ref_head[0];
#if ($ref_chr eq "ON563414.3" || $ref_chr eq "NC_003310.1") {
#	$reftag=1;
#}

`dos2unix $opts{i}`;
open IN,"<$opts{i}";
my %hash;
my $chr;
while (my $aline=<IN>) {
	chomp $aline;
	if ($aline=~/^\>/) {
		my @arr=split /\s+/,$aline;
		$arr[0]=~s/\>//;
		$chr=$arr[0];
	}else{
		$hash{$chr}.=uc($aline);
	}
}
close IN;

my $out_dir=$opts{dir};

my $log_out=$out_dir."/Statistics.txt";
open LOGOUT,">$log_out";
print LOGOUT "Ref_Version\tAcc_Version\tQuery_Length\tAlignment/NO\tSNP\tInsertion\tDeletion\tIndel\tTotal_Vars\tSimilarity\tAA_Changed_NO\tVCF_File\n";

foreach my $key (keys %hash) {
	my $query_file=$out_dir."/".$key.".fasta";
	open OUT,">$query_file";
	print OUT ">$key\n$hash{$key}\n";
	close OUT;

	my $query_length=length($hash{$key});
	my $out_prefix=$out_dir."/".$key;

	print LOGOUT "$ref_chr\t$key\t$query_length";

	`dnadiff $ref_file $query_file -p $out_prefix`;

	my $mut_file=$out_prefix.".snps";
	if (-z $mut_file) {
		my $mcoords=$out_prefix.".mcoords";
		if (-z $mcoords) {
			print LOGOUT "\tNO\n";
			`rm -rf "$out_prefix.1coords" "$out_prefix.1delta" "$out_prefix.mcoords" "$out_prefix.mdelta" "$out_prefix.qdiff" "$out_prefix.rdiff" "$out_prefix.report" "$out_prefix.delta" "$out_prefix.unqry" "$out_prefix.unref" "$out_prefix.snps"`;
		}else{
			open MC,"<$mcoords";
			my $mcline=<MC>;
			chomp $mcline;
			my @arr=split /\t/,$mcline;
			print LOGOUT "\tYes\t0\t0\t0\t0\t0\t100.00%\t0\n";
			`rm -rf "$out_prefix.1coords" "$out_prefix.1delta" "$out_prefix.mcoords" "$out_prefix.mdelta" "$out_prefix.qdiff" "$out_prefix.rdiff" "$out_prefix.report" "$out_prefix.delta" "$out_prefix.snps"`;
		}
	}else{

		print LOGOUT "\tYes";
		my $vcf_tmp_out=$out_prefix."_tmp.vcf";
		open VCFTOUT,">$vcf_tmp_out";
		print VCFTOUT "##fileformat=VCFv4.2\n";
		print VCFTOUT "#CHROM\tPOS\tID\tREF\tALT\tQUAL\tFILTER\tINFO\tFORMAT\t$key\n";

		open MF,"<$mut_file";
		my $content=do {local $/=undef; <MF>;};
		my @line=split /\n/,$content;

		my $total_var=0;
		my $start;
		my $end;
		my $ref;
		my $alt;
		my $type;
		my %sta;
		my %mut_pos;

		#my %gff_line;
		my %tab_line;
		my %vcf_line;
		my @gff_id;

		for (my $i=0;$i<=$#line ;$i++) {
			my @arr=split /\t/,$line[$i];

			$start=$arr[0];
			$end=$arr[0];
			$ref=$arr[1];
			$alt=$arr[2];
			$type="SNP";
			if ($arr[1] eq ".") {
				$type="Insertion";
				my $pos=$start-1;
				$ref=substr($ref_seq,$pos-1,1);
				if ($i==$#line) {
					$mut_pos{$start}=1;
					$total_var++;
					print VCFTOUT "$ref_chr\t$pos\t$pos:$ref>$ref$alt\t$ref\t$ref$alt\t.\t.\t.\tGT\t1\n";
					$vcf_line{"$pos:$ref>$ref$alt"}="$ref_chr\t$pos\t$pos:$ref>$ref$alt\t$ref\t$ref$alt\t.\t.\t.";
					$tab_line{"$pos:$ref>$ref$alt"}="$pos\t$type\t$pos\t$pos\t$ref\t$ref$alt";

					push @gff_id,"$pos:$ref>$ref$alt";
					$sta{$type}++;
					last;
				}
				for (my $j=$i+1;$j<=$#line ;$j++) {
					my @brr=split /\t/,$line[$j];
					if ($brr[1] ne ".") {
						$mut_pos{$start}=1;
						$total_var++;
						print VCFTOUT "$ref_chr\t$pos\t$pos:$ref>$ref$alt\t$ref\t$ref$alt\t.\t.\t.\tGT\t1\n";
						$vcf_line{"$pos:$ref>$ref$alt"}="$ref_chr\t$pos\t$pos:$ref>$ref$alt\t$ref\t$ref$alt\t.\t.\t.";
						$tab_line{"$pos:$ref>$ref$alt"}="$pos\t$type\t$pos\t$pos\t$ref\t$ref$alt";
						push @gff_id,"$pos:$ref>$ref$alt";
						$sta{$type}++;
						$i=$j-1;
						last;
					}else{
						if ($brr[0]==$start) {
							$alt.=$brr[2];
							if ($j==$#line) {
								$mut_pos{$start}=1;
								$total_var++;
								print VCFTOUT "$ref_chr\t$pos\t$pos:$ref>$ref$alt\t$ref\t$ref$alt\t.\t.\t.\tGT\t1\n";
								$vcf_line{"$pos:$ref>$ref$alt"}="$ref_chr\t$pos\t$pos:$ref>$ref$alt\t$ref\t$ref$alt\t.\t.\t.";
								$tab_line{"$pos:$ref>$ref$alt"}="$pos\t$type\t$pos\t$pos\t$ref\t$ref$alt";
								push @gff_id,"$pos:$ref>$ref$alt";
								$sta{$type}++;
								$i=$j;
							}
						}else{
							$mut_pos{$start}=1;
							$total_var++;
							print VCFTOUT "$ref_chr\t$pos\t$pos:$ref>$ref$alt\t$ref\t$ref$alt\t.\t.\t.\tGT\t1\n";
							$vcf_line{"$pos:$ref>$ref$alt"}="$ref_chr\t$pos\t$pos:$ref>$ref$alt\t$ref\t$ref$alt\t.\t.\t.";
							$tab_line{"$pos:$ref>$ref$alt"}="$pos\t$type\t$pos\t$pos\t$ref\t$ref$alt";
							push @gff_id,"$pos:$ref>$ref$alt";
							$sta{$type}++;
							$i=$j-1;
							last;
						}
						
					}
				}
			}else{
				if ($arr[2] eq ".") {
					$type="Deletion";
					$alt="-";
					for (my $j=$i+1;$j<=$#line ;$j++) {
						my @brr=split /\t/,$line[$j];
						if ($brr[2] ne ".") {
							$mut_pos{$start}=1;
							$total_var++;
							print VCFTOUT "$ref_chr\t$start\t$start:$ref>$alt\t$ref\t$alt\t.\t.\t.\tGT\t1\n";
							$vcf_line{"$start:$ref>$alt"}="$ref_chr\t$start\t$start:$ref>$alt\t$ref\t$alt\t.\t.\t.";
							$tab_line{"$start:$ref>$alt"}="$start\t$type\t$start\t$end\t$ref\t$alt";
							push @gff_id,"$start:$ref>$alt";
							$sta{$type}++;
							$i=$j-1;
							last;
						}else{
							if ($brr[0]-$end==1) {
								$end=$brr[0];
								$ref.=$brr[1];
								if ($j==$#line) {
									$mut_pos{$start}=1;
									$total_var++;
									print VCFTOUT "$ref_chr\t$start\t$start:$ref>$alt\t$ref\t$alt\t.\t.\t.\tGT\t1\n";
									$vcf_line{"$start:$ref>$alt"}="$ref_chr\t$start\t$start:$ref>$alt\t$ref\t$alt\t.\t.\t.";
									$tab_line{"$start:$ref>$alt"}="$start\t$type\t$start\t$end\t$ref\t$alt";
									push @gff_id,"$start:$ref>$alt";
									$sta{$type}++;
									$i=$j;
								}
							}else{
								$mut_pos{$start}=1;
								$total_var++;
								print VCFTOUT "$ref_chr\t$start\t$start:$ref>$alt\t$ref\t$alt\t.\t.\t.\tGT\t1\n";
								$vcf_line{"$start:$ref>$alt"}="$ref_chr\t$start\t$start:$ref>$alt\t$ref\t$alt\t.\t.\t.";
								$tab_line{"$start:$ref>$alt"}="$start\t$type\t$start\t$end\t$ref\t$alt";
								push @gff_id,"$start:$ref>$alt";
								$sta{$type}++;
								$i=$j-1;
								last;
							}
						}
					}
				}else{
					for (my $j=$i+1;$j<=$#line ;$j++) {
						my @brr=split /\t/,$line[$j];
						if ($brr[0]-$end==1) {
							if ($brr[1] ne ".") {
								$ref.=$brr[1];
								$end=$brr[0];
							}
							$alt.=$brr[2];
							if ($j==$#line) {
								$total_var++;
								$mut_pos{$start}=1;
								print VCFTOUT "$ref_chr\t$start\t$start:$ref>$alt\t$ref\t$alt\t.\t.\t.\tGT\t1\n";
								$vcf_line{"$start:$ref>$alt"}="$ref_chr\t$start\t$start:$ref>$alt\t$ref\t$alt\t.\t.\t.";
								$tab_line{"$start:$ref>$alt"}="$start\t$type\t$start\t$end\t$ref\t$alt";
								push @gff_id,"$start:$ref>$alt";
								$sta{$type}++;
								$i=$j;
							}
						}else{
							if (length($ref)==1 && length($alt)==1) {
								$type="SNP";
								$total_var++;
								$mut_pos{$start}=1;
								print VCFTOUT "$ref_chr\t$start\t$start:$ref>$alt\t$ref\t$alt\t.\t.\t.\tGT\t1\n";
								$vcf_line{"$start:$ref>$alt"}="$ref_chr\t$start\t$start:$ref>$alt\t$ref\t$alt\t.\t.\t.";
								$tab_line{"$start:$ref>$alt"}="$start\t$type\t$start\t$end\t$ref\t$alt";
								push @gff_id,"$start:$ref>$alt";
								$sta{$type}++;
							}else{
								$type="Indel";
								$mut_pos{$start}=1;
								$total_var++;
								print VCFTOUT "$ref_chr\t$start\t$start:$ref>$alt\t$ref\t$alt\t.\t.\t.\tGT\t1\n";
								$vcf_line{"$start:$ref>$alt"}="$ref_chr\t$start\t$start:$ref>$alt\t$ref\t$alt\t.\t.\t.";
								$tab_line{"$start:$ref>$alt"}="$start\t$type\t$start\t$end\t$ref\t$alt";
								push @gff_id,"$start:$ref>$alt";
								$sta{$type}++;
							}
							$i=$j-1;
							last;
						}
					}
				}
			}
			if ($i==$#line) {
				if (!defined $mut_pos{$start}) {
					$mut_pos{$start}=1;
					$total_var++;
					print VCFTOUT "$ref_chr\t$start\t$start:$ref>$alt\t$ref\t$alt\t.\t.\t.\tGT\t1\n";
					$vcf_line{"$start:$ref>$alt"}="$ref_chr\t$start\t$start:$ref>$alt\t$ref\t$alt\t.\t.\t.";
					$tab_line{"$start:$ref>$alt"}="$start\t$type\t$start\t$end\t$ref\t$alt";
					push @gff_id,"$start:$ref>$alt";
					$sta{$type}++;
				}
			}
		}
		close MF;
		close VCFTOUT;

		if (!defined $sta{"SNP"}) {
			$sta{"SNP"}=0;
		}
		if (!defined $sta{"Insertion"}) {
			$sta{"Insertion"}=0;
		}
		if (!defined $sta{"Deletion"}) {
			$sta{"Deletion"}=0;
		}
		if (!defined $sta{"Indel"}) {
			$sta{"Indel"}=0;
		}

		my $Similarity=(1-2*$total_var/($ref_length+$query_length))*100;
		$Similarity=sprintf "%.2f",$Similarity;

		my $vcffile=$key.".vcf";
		my $aa_change=0;


		if ($total_var==0) {
			$vcffile="";
		}else{
			my $vcf_out=$out_prefix.".vcf";
			#if ($reftag==1) {
				my $vep_out=$out_prefix.".vep";
				`export PERL5LIB=\$PERL5LIB`;
				`vep -offline -i $vcf_tmp_out -o $vep_out --species Monkeypox --cache_version 1 --dir $opts{refdir} --distance 0 --force`;

				my %vep_anno;
				open VEP,"<$vep_out";
				while (my $aline=<VEP>) {
					next if($aline=~/^\#/);
					chomp $aline;
					my @arr=split /\t/,$aline;
					$arr[6]=~s/3_prime_UTR_variant//g;
					$arr[6]=~s/5_prime_UTR_variant//g;
					$arr[6]=~s/,,/,/g;
					if ($arr[6]=~/^,/) {
						$arr[6]=~s/,//;
					}
					if ($arr[6]=~/,$/) {
						chop $arr[6];
					}

					if ($arr[6]=~/intergenic_variant/) {
						$vep_anno{$arr[0]}.=";ANN=$arr[6]";
					}elsif ($arr[6]=~/stream/) {
						$arr[13]=~/DISTANCE=(\d+);/;
						my $dist=$1;
						$vep_anno{$arr[0]}.=";ANN=$arr[6],DISTANCE=$dist,$arr[3]";
					}else{
						my ($cc,$pp);
						if ($arr[10] eq "-") {
							$vep_anno{$arr[0]}.=";ANN=$arr[6]";
						}else{
							my @brr=split /:/,$arr[1];
							$arr[11]=~s/\//$brr[1]/;
							$cc=$arr[3].":c.".$arr[11];
							if ($arr[10]!~/\//) {
								$pp=$arr[3].":p.".$arr[10].$arr[9];
							}else{
								$aa_change++;
								$arr[10]=~s/\//$arr[9]/;
								$pp=$arr[3].":p.".$arr[10];
							}
							$vep_anno{$arr[0]}.=";ANN=$arr[6],$pp,$cc";
						}
					}
				}
				close VEP;

				open VCF,">$vcf_out";
				print VCF "##fileformat=VCFv4.2\n";
				print VCF "##ANN: Annotation detail by VEP\n";
				print VCF "#CHROM\tPOS\tID\tREF\tALT\tQUAL\tFILTER\tINFO\tANN\t$key\n";

				for (my $i=0;$i<=$#gff_id ;$i++) {
					$vep_anno{$gff_id[$i]}=~s/;//;
					print VCF "$vcf_line{$gff_id[$i]}\t$vep_anno{$gff_id[$i]}\t1\n";
				}
				close VCF;

				`rm -rf $vep_out "$out_prefix.vep_summary.html" $vcf_tmp_out`;
			#}else{
			#	$aa_change="NULL";
			#	`mv $vcf_tmp_out $vcf_out`;
			#}
		}
		print LOGOUT "\t$sta{\"SNP\"}\t$sta{\"Insertion\"}\t$sta{\"Deletion\"}\t$sta{\"Indel\"}\t$total_var\t$Similarity%\t$aa_change\t$vcffile\n";

		`rm -rf "$out_prefix.1coords" "$out_prefix.1delta" "$out_prefix.mcoords" "$out_prefix.mdelta" "$out_prefix.qdiff" "$out_prefix.rdiff" "$out_prefix.report" "$out_prefix.delta" "$out_prefix.snps" $vcf_tmp_out`;
	}
	`rm -rf $query_file`;
}

close LOGOUT;



sub usage{
print <<"USAGE";
Description: ﶻ߱ע.
Usage:
$0 -r -refdir -i -dir
options:
-r input reference fasta file
-r input formatted reference annotation directory
-i input virus query file
-dir output directory file
-h help
USAGE
exit(1);
}
